@@ -20,16 +20,19 @@ import imp
 import logging
 import os
 
+# Need to do this before any tests or virtinst import
 os.environ["VIRTINST_TEST_SUITE"] = "1"
 os.environ["VIRTINST_TEST_URL_DIR"] = os.path.abspath(
     "tests/cli-test-xml/fakefedoratree/")
 
+# pylint: disable=wrong-import-position
 from virtcli import cliconfig
 # This sets all the cli bits back to their defaults
 reload(cliconfig)
 
 from tests import utils
 
+
 # Setup logging
 rootLogger = logging.getLogger()
 for handler in rootLogger.handlers:
@@ -65,6 +68,3 @@ virtxml = _import("virtxml", "virt-xml")
 # Variable used to store a local iso or dir path to check for a distro
 # Specified via 'python setup.py test_urls --path"
 URLTEST_LOCAL_MEDIA = []
-
-# Used to implement test_initrd_inject --distro
-INITRD_TEST_DISTROS = []
