@@ -165,6 +165,7 @@ class XMLParseTest(unittest.TestCase):
         check("vmport", False, True)
         check("kvm_hidden", None, True)
         check("pvspinlock", None, True)
+        check("gic_version", None, False)
 
         check = self._make_checker(guest.cpu)
         check("match", "exact", "strict")
@@ -227,7 +228,7 @@ class XMLParseTest(unittest.TestCase):
     def testSeclabel(self):
         guest, outfile = self._get_test_content("change-seclabel")
 
-        check = self._make_checker(guest.seclabel[0])
+        check = self._make_checker(guest.seclabels[0])
         check("type", "static", "none")
         check("model", "selinux", "apparmor")
         check("label", "foolabel", "barlabel")
@@ -235,7 +236,7 @@ class XMLParseTest(unittest.TestCase):
         check("baselabel", None, "baselabel")
         check("relabel", None, False)
 
-        guest.remove_child(guest.seclabel[1])
+        guest.remove_child(guest.seclabels[1])
 
         self._alter_compare(guest.get_xml_config(), outfile)
 
@@ -258,7 +259,7 @@ class XMLParseTest(unittest.TestCase):
         seclabel.type = "static"
         seclabel.label = "frob"
         self.assertTrue(
-            guest.seclabel[0].get_xml_config().startswith("<seclabel"))
+            guest.seclabels[0].get_xml_config().startswith("<seclabel"))
 
         check = self._make_checker(guest.cpu)
         check("model", None, "foobar")
@@ -349,9 +350,10 @@ class XMLParseTest(unittest.TestCase):
         check("removable", None, False, True)
 
         disk = guest.get_devices("disk")[1]
-        check = self._make_checker(disk.seclabel[1])
+        check = self._make_checker(disk.seclabels[1])
         check("model", "dac")
-        check("type", "dynamic", "none")
+        check("relabel", None, True)
+        check("label", None, "foo-my-label")
 
         disk = _get_disk("hdc")
         check = self._make_checker(disk)
@@ -649,6 +651,7 @@ class XMLParseTest(unittest.TestCase):
         check("clipboard_copypaste", None, True)
         check("mouse_mode", None, "client")
         check("filetransfer_enable", None, False)
+        check("gl", None, True)
 
         self._alter_compare(guest.get_xml_config(), outfile)
 
@@ -673,6 +676,7 @@ class XMLParseTest(unittest.TestCase):
         check("model", "cirrus", "cirrus", "qxl")
         check("ram", None, 100)
         check("vgamem", None, 8192)
+        check("accel3d", None, True)
 
         self._alter_compare(guest.get_xml_config(), outfile)
 
